@@ -104,11 +104,11 @@ static igraph_error_t igraph_i_community_spinglass_negative(
  * with positive and negative links, http://arxiv.org/abs/0811.2329 .
  *
  * \param graph The input graph, it may be directed but the direction
- *     of the edges is not used in the algorithm.
+ *     of the edges is ignored by the algorithm.
  * \param weights The vector giving the edge weights, it may be \c NULL,
  *     in which case all edges are weighted equally. The edge weights
  *     must be positive unless using the \c IGRAPH_SPINCOMM_IMP_NEG
- *     implementation. This condition is not verified by the function.
+ *     implementation.
  * \param modularity Pointer to a real number, if not \c NULL then the
  *     modularity score of the solution will be stored here. This is the
  *     gereralized modularity that simplifies to the one defined in
@@ -126,20 +126,17 @@ static igraph_error_t igraph_i_community_spinglass_negative(
  *     NULL then the sizes of the clusters will stored here in cluster
  *     number order. The vector will be resized as needed.
  * \param spins Integer giving the number of spins, i.e. the maximum
- *     number of clusters. Usually it is not a program to give a high
- *     number here, the default was 25 in the original code. Even if
- *     the number of spins is high the number of clusters in the
- *     result might be small.
+ *     number of clusters. Even if the number of spins is high the number of
+ *     clusters in the result might be small.
  * \param parupdate A logical constant, whether to update all spins in
- *     parallel. The default for this argument was \c false in
- *     the original code. It is not implemented in the
- *     \c IGRAPH_SPINCOMM_INP_NEG implementation.
- * \param starttemp Real number, the temperature at the start. The
- *     value of this argument was 1.0 in the original code.
- * \param stoptemp Real number, the algorithm stops at this
- *     temperature. The default was 0.01 in the original code.
+ *     parallel. It is not implemented in the \c IGRAPH_SPINCOMM_INP_NEG
+ *     implementation.
+ * \param starttemp Real number, the temperature at the start. A reasonable
+ *     default is 1.0.
+ * \param stoptemp Real number, the algorithm stops at this temperature. A
+ *     reasonable default is 0.01.
  * \param coolfact Real number, the cooling factor for the simulated
- *     annealing. The default was 0.99 in the original code.
+ *     annealing. A reasonable default is 0.99.
  * \param update_rule The type of the update rule. Possible values: \c
  *     IGRAPH_SPINCOMM_UPDATE_SIMPLE and \c
  *     IGRAPH_SPINCOMM_UPDATE_CONFIG. Basically this parameter defines
@@ -149,29 +146,20 @@ static igraph_error_t igraph_i_community_spinglass_negative(
  *     configuration model is used. The configuration means that the
  *     baseline for the clustering is a random graph with the same
  *     degree distribution as the input graph.
- * \param gamma Real number. The gamma parameter of the
- *     algorithm. This defines the weight of the missing and existing
- *     links in the quality function for the clustering. The default
- *     value in the original code was 1.0, which is equal weight to
- *     missing and existing edges. Smaller values make the existing
- *     links contibute more to the energy function which is minimized
- *     in the algorithm. Bigger values make the missing links more
- *     important. (If my understanding is correct.)
+ * \param gamma Real number. The gamma parameter of the algorithm,
+ *     acting as a resolution parameter. Smaller values typically lead to
+ *     larger clusters, larger values typically lead to smaller clusters.
  * \param implementation Constant, chooses between the two
  *     implementations of the spin-glass algorithm that are included
  *     in igraph. \c IGRAPH_SPINCOMM_IMP_ORIG selects the original
  *     implementation, this is faster, \c IGRAPH_SPINCOMM_INP_NEG selects
- *     a new implementation by Vincent Traag that allows negative edge
- *     weights.
- * \param gamma_minus Real number. Parameter for the \c
- *     IGRAPH_SPINCOMM_IMP_NEG implementation. This
- *     specifies the balance between the importance of present and
- *     non-present negative weighted edges in a community. Smaller values of
- *     \p gamma_minus lead to communities with lesser
- *     negative intra-connectivity.
- *     If this argument is set to zero, the algorithm reduces to a graph
- *     coloring algorithm, using the number of spins as the number of
- *     colors.
+ *     an implementation that allows negative edge weights.
+ * \param gamma_minus Real number. Parameter for the \c IGRAPH_SPINCOMM_IMP_NEG
+ *     implementation. This acts as a resolution parameter for the negative part
+ *     of the network. Smaller values of \p gamma_minus leads to fewer negative
+ *     edges within clusters. If this argument is set to zero, the algorithm
+ *     reduces to a graph coloring algorithm when all edges have negative
+ *     weights, using the number of spins as the number of colors.
  * \return Error code.
  *
  * \sa igraph_community_spinglass_single() for calculating the community
@@ -256,6 +244,9 @@ static igraph_error_t igraph_i_community_spinglass_orig(
             IGRAPH_ERROR("Invalid weight vector length", IGRAPH_EINVAL);
         }
         use_weights = 1;
+        if (igraph_vector_min(weights) < 0) {
+            IGRAPH_ERROR("Weights must not be negative when using the original implementation of spinglass communities. Select the implementation meant for negative weights.", IGRAPH_EINVAL);
+        }
     }
     if (coolfact < 0 || coolfact >= 1.0) {
         IGRAPH_ERROR("Invalid cooling factor", IGRAPH_EINVAL);
