@@ -57,10 +57,9 @@ static igraph_error_t igraph_i_average_path_length_unweighted(
 
     *res = 0;
     already_added = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
-    if (already_added == 0) {
-        IGRAPH_ERROR("Average path length calculation failed", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for average path length.");
     IGRAPH_FINALLY(igraph_free, already_added);
+
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, 100);
 
     IGRAPH_CHECK(igraph_adjlist_init(
@@ -241,7 +240,9 @@ static igraph_error_t igraph_i_average_path_length_dijkstra(
                 igraph_bool_t active = igraph_2wheap_has_active(&Q, tto);
                 igraph_bool_t has = igraph_2wheap_has_elem(&Q, tto);
                 igraph_real_t curdist = active ? -igraph_2wheap_get(&Q, tto) : 0.0;
-                if (!has) {
+                if (altdist == IGRAPH_INFINITY) {
+                    /* Ignore edges with positive infinite weight */
+                } else if (!has) {
                     /* This is the first non-infinite distance */
                     IGRAPH_CHECK(igraph_2wheap_push_with_index(&Q, tto, -altdist));
                 } else if (altdist < curdist) {
@@ -343,7 +344,8 @@ igraph_error_t igraph_average_path_length(const igraph_t *graph,
  *       non-negative for Dijkstra's algorithm to work. Additionally, no
  *       edge weight may be NaN. If either case does not hold, an error
  *       is returned. If this is a null pointer, then the unweighted
- *       version, \ref igraph_average_path_length() is called.
+ *       version, \ref igraph_average_path_length() is called. Edges with positive
+ *       infinite weight are ignored.
  * \param directed Boolean, whether to consider directed paths.
  *    Ignored for undirected graphs.
  * \param unconn If \c true, only those pairs are considered for the calculation
@@ -400,7 +402,8 @@ igraph_error_t igraph_average_path_length_dijkstra(const igraph_t *graph,
  *       edge weight may be NaN. If either case does not hold, an error
  *       is returned. If this is a null pointer, then the unweighted
  *       version, \ref igraph_average_path_length() is used in calculating
- *       the global efficiency.
+ *       the global efficiency. Edges with positive infinite weights are
+ *       ignored.
  * \param directed Boolean, whether to consider directed paths.
  *    Ignored for undirected graphs.
  * \return Error code:
@@ -675,7 +678,8 @@ static igraph_error_t igraph_i_local_efficiency_dijkstra(
  *       non-negative. Additionally, no edge weight may be NaN. If either
  *       case does not hold, an error is returned. If this is a null
  *       pointer, then the unweighted version,
- *       \ref igraph_average_path_length() is called.
+ *       \ref igraph_average_path_length() is called. Edges with positive
+ *       infinite weights are ignored.
  * \param directed Boolean, whether to consider directed paths.
  *    Ignored for undirected graphs.
  * \param mode How to determine the local neighborhood of each vertex
@@ -743,9 +747,7 @@ igraph_error_t igraph_local_efficiency(const igraph_t *graph, igraph_vector_t *r
         igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
 
         already_counted = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
-        if (already_counted == 0) {
-            IGRAPH_ERROR("Local efficiency calculation failed", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-        }
+        IGRAPH_CHECK_OOM(already_counted, "Insufficient memory for local efficiency calculation.");
         IGRAPH_FINALLY(igraph_free, already_counted);
 
         IGRAPH_CHECK(igraph_adjlist_init(
@@ -778,15 +780,15 @@ igraph_error_t igraph_local_efficiency(const igraph_t *graph, igraph_vector_t *r
         igraph_2wheap_t Q;
 
         if (igraph_vector_size(weights) != no_of_edges) {
-            IGRAPH_ERROR("Weight vector length does not match the number of edges", IGRAPH_EINVAL);
+            IGRAPH_ERROR("Weight vector length does not match the number of edges.", IGRAPH_EINVAL);
         }
         if (no_of_edges > 0) {
             igraph_real_t min = igraph_vector_min(weights);
             if (min < 0) {
-                IGRAPH_ERROR("Weight vector must be non-negative", IGRAPH_EINVAL);
+                IGRAPH_ERRORF("Weights must not be negative, got %g.", IGRAPH_EINVAL, min);
             }
             else if (isnan(min)) {
-                IGRAPH_ERROR("Weight vector must not contain NaN values", IGRAPH_EINVAL);
+                IGRAPH_ERROR("Weights must not contain NaN values.", IGRAPH_EINVAL);
             }
         }
 
@@ -831,7 +833,8 @@ igraph_error_t igraph_local_efficiency(const igraph_t *graph, igraph_vector_t *r
  * \param graph The graph object.
  * \param res Pointer to a real number, this will contain the result.
  * \param weights The edge weights. They must be all non-negative.
- *    If a null pointer is given, all weights are assumed to be 1.
+ *    If a null pointer is given, all weights are assumed to be 1. Edges
+ *    with positive infinite weight are ignored.
  * \param directed Boolean, whether to consider directed paths.
  *    Ignored for undirected graphs.
  * \param mode How to determine the local neighborhood of each vertex
@@ -981,10 +984,9 @@ igraph_error_t igraph_diameter(const igraph_t *graph, igraph_real_t *res,
         dirmode = IGRAPH_ALL;
     }
     already_added = IGRAPH_CALLOC(no_of_nodes, igraph_integer_t);
-    if (already_added == 0) {
-        IGRAPH_ERROR("diameter failed", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for diameter calculation.");
     IGRAPH_FINALLY(igraph_free, already_added);
+
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, 100);
 
     IGRAPH_CHECK(igraph_adjlist_init(graph, &allneis, dirmode, IGRAPH_LOOPS, IGRAPH_MULTIPLE));
@@ -1077,7 +1079,7 @@ igraph_error_t igraph_diameter(const igraph_t *graph, igraph_real_t *res,
  *
  * \param graph The input graph, can be directed or undirected.
  * \param weights The edge weights of the graph. Can be \c NULL for an
- *        unweighted graph.
+ *        unweighted graph. Edges with positive infinite weight are ignored.
  * \param res Pointer to a real number, if not \c NULL then it will contain
  *        the diameter (the actual distance).
  * \param from Pointer to an integer, if not \c NULL it will be set to the
